@@ -88,12 +88,6 @@
 ;;; 	        (char* read_dir (DIR*)))
 ;;;   "" '("sys/types.h" "dirent.h"))
 ;;; 
-;;; (let ((dir (opendir "/home/bil/gtk-snd")))
-;;;   (do ((p (read_dir dir) (read_dir dir)))
-;;;       ((= (length p) 0))
-;;;     (format *stderr* "~A " p))
-;;;   (closedir dir))
-;;;
 ;;; (define (memory-usage)
 ;;;   (with-let *libc*
 ;;;     (let ((v (rusage.make))) 
@@ -103,11 +97,13 @@
 ;;;         (* 1024 mem)))))
 ;;; --------------------------------------------------------------------------------
 
-(define *cload-cflags* "") 
+(define *cload-cflags* (if (provided? 'clang) "-fPIC" ""))
 (define *cload-ldflags* "")
 (if (not (defined? '*cload-directory*))
     (define *cload-directory* ""))
 
+(define *cload-c-compiler* (if (provided? 'gcc) "gcc" (if (provided? 'clang) "clang" "cc")))
+
 
 (define-macro (defvar name value) 
   `(if (not (defined? ',name)) 
@@ -206,7 +202,11 @@
 
   (set! c-define-output-file-counter (+ c-define-output-file-counter 1))
 
-  (let ((file-name (string-append *cload-directory* (or output-name (format #f "temp-s7-output-~D" c-define-output-file-counter)))))
+  (let ((file-name (string-append *cload-directory* 
+				  (if (and (> (length *cload-directory*) 0)
+					   (not (char=? (string-ref *cload-directory* (- (length *cload-directory*) 1)) #\/)))
+				      "/" "")
+				  (or output-name (format #f "temp-s7-output-~D" c-define-output-file-counter)))))
     (let ((c-file-name (string-append file-name ".c"))
 	  (o-file-name (string-append file-name ".o"))
 	  (so-file-name (string-append file-name ".so"))
@@ -295,7 +295,7 @@
 	;; C function -> scheme
 	(let ((func-name (symbol->string (collides? name))))
 	  (let ((num-args (length arg-types))
-		(base-name (string-append (if (> (length prefix) 0) prefix "s7_dl") "_" func-name)) ; not "g" -- collides with glib
+		(base-name (string-append (if (> (length prefix) 0) prefix "s7_") "_" func-name)) ; not "g" -- collides with glib
 		(scheme-name (string-append prefix (if (> (length prefix) 0) ":" "") func-name)))
 	  
 	    (if (and (= num-args 1) 
@@ -324,11 +324,10 @@
 		  (if (eq? true-type 's7_pointer)
 		      (format p "    ~A_~D = s7_car(arg);~%" base-name i)
 		      (if (eq? s7-type 'c_pointer)
-			  (begin
-			    (format p "  if (s7_is_c_pointer_of_type(s7_car(arg), s7_make_symbol(sc, ~S)))~%" (symbol->string nominal-type))
-			    (format p "    ~A_~D = (~A)s7_c_pointer(s7_car(arg));~%" base-name i nominal-type)
-			    (format p "  else return(s7_wrong_type_arg_error(sc, ~S, ~D, s7_car(arg), ~S));~%"
-				func-name (if (= num-args 1) 0 (+ i 1)) (symbol->string nominal-type)))
+			  (format p "    ~A_~D = (~A)s7_c_pointer_with_type(sc, s7_car(arg), s7_make_symbol(sc, ~S), __func__, ~S);~%" 
+				  base-name i nominal-type
+				  (symbol->string nominal-type)
+				  (if (= num-args 1) 0 (+ i 1)))
 			  (begin
 			    (format p "  if (~A(s7_car(arg)))~%" (checker true-type))
 			    (format p "    ~A_~D = (~A)~A(~As7_car(arg));~%"
@@ -337,8 +336,7 @@
 				    (s7->C true-type)                               ; s7_number_to_real which requires 
 				    (if (memq s7-type '(boolean real))              ;   the extra sc arg
 					"sc, " ""))
-			    (format p "  else return(s7_wrong_type_arg_error(sc, ~S, ~D, s7_car(arg), ~S));~%"
-				    func-name 
+			    (format p "  else return(s7_wrong_type_arg_error(sc, __func__, ~D, s7_car(arg), ~S));~%"
 				    (if (= num-args 1) 0 (+ i 1))
 				    (if (symbol? s7-type) 
 					(symbol->string s7-type) 
@@ -490,7 +488,7 @@
 	   (format p "  ~A~%" init-str))
 	 (reverse inits))
 	
-	;; "constants" -- actually variables in s7 because we want them to be local to the current environment (this comment is obsolete)
+	;; constants
 	(if (pair? constants)
 	    (begin
 	      (format p "~%")
@@ -563,14 +561,16 @@
 	  (format p "~%  /* double optimizer connections */~%")
 	  (for-each
 	   (lambda (f)
-	     (format p "  s7_set~A_function(s7_name_to_value(sc, ~S), ~A~A);~%" (caddr f) (cadr f) (car f) (caddr f)))
+	     (unless (defined? (symbol (cadr f)) (rootlet))
+	       (format p "  s7_set~A_function(sc, s7_name_to_value(sc, ~S), ~A~A);~%" (caddr f) (cadr f) (car f) (caddr f))))
 	   double-funcs))
 	
 	(when (pair? int-funcs)
 	  (format p "~%  /* int optimizer connections */~%")
 	  (for-each
 	   (lambda (f)
-	     (format p "  s7_set~A_function(s7_name_to_value(sc, ~S), ~A~A);~%" (caddr f) (cadr f) (car f) (caddr f)))
+	     (unless (defined? (symbol (cadr f)) (rootlet))
+	       (format p "  s7_set~A_function(sc, s7_name_to_value(sc, ~S), ~A~A);~%" (caddr f) (cadr f) (car f) (caddr f))))
 	   int-funcs))
 	
 	(format p "}~%")
@@ -580,10 +580,10 @@
 	
 	(cond ((provided? 'osx)
 	       ;; I assume the caller is also compiled with these flags?
-	       (system (format #f "gcc -c ~A -o ~A ~A ~A" 
-			       c-file-name o-file-name *cload-cflags* cflags))
-	       (system (format #f "gcc ~A -o ~A -dynamic -bundle -undefined suppress -flat_namespace ~A ~A" 
-			       o-file-name so-file-name *cload-ldflags* ldflags)))
+	       (system (format #f "~A -c ~A -o ~A ~A ~A" 
+			       *cload-c-compiler* c-file-name o-file-name *cload-cflags* cflags))
+	       (system (format #f "~A ~A -o ~A -dynamic -bundle -undefined suppress -flat_namespace ~A ~A" 
+			       *cload-c-compiler* o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
 	      ((provided? 'freebsd)
 	       (system (format #f "cc -fPIC -c ~A -o ~A ~A ~A" 
@@ -592,10 +592,10 @@
 			       o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
 	      ((provided? 'openbsd)
-	       (system (format #f "gcc -fPIC -ftrampolines -c ~A -o ~A ~A ~A" 
-			       c-file-name o-file-name *cload-cflags* cflags))
-	       (system (format #f "gcc ~A -shared -o ~A ~A ~A" 
-			       o-file-name so-file-name *cload-ldflags* ldflags)))
+	       (system (format #f "~A -fPIC -ftrampolines -c ~A -o ~A ~A ~A" 
+			       *cload-c-compiler* c-file-name o-file-name *cload-cflags* cflags))
+	       (system (format #f "~A ~A -shared -o ~A ~A ~A" 
+			       *cload-c-compiler* o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
 	      ((provided? 'sunpro_c) ; just guessing here...
 	       (system (format #f "cc -c ~A -o ~A ~A ~A" 
@@ -603,13 +603,11 @@
 	       (system (format #f "cc ~A -G -o ~A ~A ~A" 
 			       o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
-	      ;; what about clang?  Maybe use cc below, not gcc (and in osx case above)
-	      
 	      (else
-	       (system (format #f "gcc -fPIC -c ~A -o ~A ~A ~A" 
-			       c-file-name o-file-name *cload-cflags* cflags))
-	       (system (format #f "gcc ~A -shared -o ~A ~A ~A" 
-			       o-file-name so-file-name *cload-ldflags* ldflags)))))
+	       (system (format #f "~A -fPIC -c ~A -o ~A ~A ~A" 
+			       *cload-c-compiler* c-file-name o-file-name *cload-cflags* cflags))
+	       (system (format #f "~A ~A -shared -o ~A ~A ~A" 
+			       *cload-c-compiler* o-file-name so-file-name *cload-ldflags* ldflags)))))
       
       (define handle-declaration 
 	(let ()
