@@ -10,11 +10,6 @@
 	(*pretty-print-left-margin* 0)
 	(*pretty-print-cycles* #t))
 
-    (define (any? f sequence)
-      (and (pair? sequence)
-	   (or (f (car sequence))
-	       (any? f (cdr sequence)))))
-
     (define pretty-print-1
       (letrec ((messy-number (lambda (z)
 			       (if (real? z)
@@ -32,8 +27,13 @@
 	       (any-keyword? (lambda (lst)
 			       (and (pair? lst)
 				    (or (keyword? (car lst))
-					(any-keyword? (cdr lst)))))))
-	
+					(any-keyword? (cdr lst))))))
+
+	       (any-let-or-hash-table? (lambda (sequence)
+					 (and (pair? sequence)
+					      (or (let? (car sequence))
+						  (hash-table? (car sequence))
+						  (any-let-or-hash-table? (cdr sequence)))))))
 	(let ((newlines 0))
 	  
 	  (define (spaces port n) 
@@ -96,7 +96,7 @@
 	  
 	  (let ((writers 
 		 (let ((h (make-hash-table)))
-		   
+
 		   ;; -------- quote
 		   (define (w-quote obj port column)
 		     (if (not (pair? (cdr obj))) ; (quote) or (quote . 1)
@@ -477,9 +477,8 @@
 		     (for-each (lambda (field)
 				 (let ((symstr (object->string (car field))))
 				   (spaces port (+ column 2))
-				   (format port "'(~A . " symstr)
-				   (pretty-print-1 (cdr field) port (+ column 4 (length symstr)))
-				   (write-char #\) port)))
+				   (format port "'~A " symstr)
+				   (pretty-print-1 (cdr field) port (+ column 4 (length symstr)))))
 			       obj)
 		     (write-char #\) port))
 		    
@@ -491,9 +490,9 @@
 			   (display "(inlet" port)
 			   (for-each (lambda (field)
 				       (let ((symstr (symbol->string (car field))))
-					 (spaces port (+ column 5))
+					 (spaces port (+ column 2))
 					 (format port ":~A " symstr)
-					 (pretty-print-1 (cdr field) port (+ column 2 (length symstr)))))
+					 (pretty-print-1 (cdr field) port (+ column 4 (length symstr)))))
 				     obj)
 			   (write-char #\) port))))
 		    
@@ -533,10 +532,7 @@
 		    ((hash-table-ref writers (car obj)) 
 		     => (lambda (f) (f obj port column)))
 		    
-		    ((any? (lambda (p)
-			     (or (hash-table? p)
-				 (let? p)))
-			   obj)
+		    ((any-let-or-hash-table? obj)
 		     (let ((first #t))
 		       (write-char #\( port)
 		       (for-each (lambda (p)
@@ -550,6 +546,15 @@
 			    (strlen (length objstr)))
 		       (if (< (+ column strlen) *pretty-print-length*)
 			   (display objstr port)
+
+			   ;; (format port "~A~A" (cond ((pair-line-number obj) => (lambda (line) (format #f "[~D] " line))) (else "")) objstr)
+			   ;; the other case is above (writers)
+			   ;;    (lambda (f) (format port "[~D]" line) (f obj port column))
+			   ;;    with column check, line check etc
+			   ;; so, the pp/profile hook only needs two calls
+			   ;; (lambda (f obj port column) (f obj port column))
+			   ;; and here (lambda (f obj port column) (f=display obj=objstr port)?
+
 			   (let ((lstlen (length obj)))
 			     
 			     (cond ((or (infinite? lstlen)
@@ -658,9 +663,7 @@
 (define (pp obj)
   (call-with-output-string
    (lambda (p)
-     (if (keyword? obj)
-	 (display obj p)
-	 (pretty-print obj p)))))
+     ((if (keyword? obj) display pretty-print) obj p))))
 
 #|
 (define (pretty-print-all)
@@ -681,7 +684,7 @@
     (if (pair? form)
 	(if (and (symbol? (car form))
 		 (macro? (symbol->value (car form))))
-	    (expand ((eval (procedure-source (symbol->value (car form)))) form))
+	    (expand (apply macroexpand (list form)))
 	    (cons (expand (car form))
 		  (expand (cdr form))))
 	form))
